@@ -1,17 +1,18 @@
 /*
  * bjb-bijiben.c
- * Copyright (C) Pierre-Yves LUYTEN 2011 <py@luyten.fr>
- * 
+ * Copyright (C) 2011 Pierre-Yves LUYTEN <py@luyten.fr>
+ * Copyright (C) 2017 Iñigo Martínez <inigomartinez@gmail.com>
+ *
  * bijiben is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
  * Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
- * 
+ *
  * bijiben is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License along
  * with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
@@ -43,118 +44,67 @@ struct _BijibenPriv
   gboolean     first_run;
   gboolean     is_loaded;
   gboolean     new_note;
-  GQueue       *files_to_open; // paths
+  GQueue       files_to_open; // paths
 };
 
 
 G_DEFINE_TYPE (Bijiben, bijiben, GTK_TYPE_APPLICATION);
 
-static void
-bijiben_new_window_internal (Bijiben *app,
-                             GFile *file,
-                             BijiItem *item);
-
+static void     bijiben_new_window_internal (Bijiben       *self,
+                                             BijiNoteObj   *note);
+static gboolean bijiben_open_path           (Bijiben       *self,
+                                             gchar         *path,
+                                             BjbWindowBase *window);
 
 static void
-on_window_activated_cb   (BjbWindowBase *window,
-                          gboolean win_is_available,
-                          Bijiben *self)
+on_window_activated_cb (BjbWindowBase *window,
+                        gboolean       available,
+                        Bijiben       *self)
 {
   BijibenPriv *priv;
-  gchar *path;
-  BijiItem *item;
-  GList *notfound, *l;
+  GList *l, *next;
 
-  item = NULL;
   priv = self->priv;
   priv->is_loaded = TRUE;
-  notfound = NULL;
 
-  while ((path = g_queue_pop_head (priv->files_to_open)))
+  for (l = priv->files_to_open.head; l != NULL; l = next)
   {
-    item = biji_manager_get_item_at_path (priv->manager, path);
-
-    if (item != NULL)
+    next = l->next;
+    if (bijiben_open_path (self, l->data, (available ? window : NULL)))
     {
-      /* If that's a note, detach it */
-      if (BIJI_IS_NOTE_OBJ (item))
-      {
-	bijiben_new_window_for_note (G_APPLICATION (self), BIJI_NOTE_OBJ (item));
-      }
-
-      /* Else, check */
-      else
-      {
-        if (win_is_available)
-           bjb_window_base_switch_to_item (window, item);
-
-        else
-           bijiben_new_window_internal (self, NULL, item);
-      }
-      g_free (path);
-    }
-
-    else
-    {
-      notfound = g_list_prepend (notfound, path);
+      g_free (l->data);
+      g_queue_delete_link (&priv->files_to_open, l);
     }
   }
 
-  /* We just wait for next provider to be loaded.
-   * TODO should also check if all providers are loaded
-   * in order to trigger file reading. */
-  for (l = notfound; l != NULL; l=l->next)
-  {
-    g_queue_push_head (priv->files_to_open, l->data);
-  }
-
-
   /* All requested notes are loaded, but we have a new one to create...
    * This implementation is not really safe,
    * we might have loaded SOME provider(s)
    * but not the default one - more work is needed here */
-  if (notfound == NULL &&
-      priv->new_note == TRUE)
+  if (priv->new_note && g_queue_is_empty (&priv->files_to_open))
   {
+    BijiItem *item;
+
     priv->new_note = FALSE;
     item = BIJI_ITEM (biji_manager_note_new (
                         priv->manager,
                         NULL,
                         bjb_settings_get_default_location (self->priv->settings)));
-    bijiben_new_window_internal (self, NULL, item);
+    bijiben_new_window_internal (self, BIJI_NOTE_OBJ (item));
   }
 }
 
-
 static void
 bijiben_new_window_internal (Bijiben     *self,
-                             GFile       *file,
-                             BijiItem    *item)
+                             BijiNoteObj *note)
 {
   BjbWindowBase *window;
-  BijiNoteObj   *note;
-  gchar         *path;
   GList         *windows;
   gboolean       not_first_window;
 
-  note = NULL;
-
   windows = gtk_application_get_windows (GTK_APPLICATION (self));
   not_first_window = (gboolean) g_list_length (windows);
 
-  if (file != NULL)
-  {
-    path = g_file_get_parse_name (file);
-    note = BIJI_NOTE_OBJ (biji_manager_get_item_at_path (self->priv->manager, path));
-    g_free (path);
-  }
-
-  else if (item != NULL && BIJI_IS_NOTE_OBJ (item))
-  {
-    note = BIJI_NOTE_OBJ (item);
-  }
-
-
   window = BJB_WINDOW_BASE (bjb_window_base_new (note));
   g_signal_connect (window, "activated",
                     G_CALLBACK (on_window_activated_cb), self);
@@ -165,11 +115,31 @@ bijiben_new_window_internal (Bijiben     *self,
     gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_CENTER);
 }
 
+static gboolean
+bijiben_open_path (Bijiben       *self,
+                   gchar         *path,
+                   BjbWindowBase *window)
+{
+  BijiItem *item;
+
+  if (!self->priv->is_loaded)
+    return FALSE;
+
+  item = biji_manager_get_item_at_path (self->priv->manager, path);
+
+  if (BIJI_IS_NOTE_OBJ (item) || !window)
+    bijiben_new_window_internal (self, BIJI_NOTE_OBJ (item));
+  else
+    bjb_window_base_switch_to_item (window, item);
+
+  return TRUE;
+}
+
 void
 bijiben_new_window_for_note (GApplication *app,
                              BijiNoteObj *note)
 {
-  bijiben_new_window_internal (BIJIBEN_APPLICATION (app), NULL, BIJI_ITEM (note));
+  bijiben_new_window_internal (BIJIBEN_APPLICATION (app), note);
 }
 
 static void
@@ -193,16 +163,17 @@ bijiben_open (GApplication  *application,
 {
   Bijiben *self;
   gint i;
+  gchar *path;
 
   self = BIJIBEN_APPLICATION (application);
 
   for (i = 0; i < n_files; i++)
   {
-    if (self->priv->is_loaded == TRUE)
-      bijiben_new_window_internal (self, files[i], NULL);
-
+    path = g_file_get_parse_name (files[i]);
+    if (bijiben_open_path (self, path, NULL))
+      g_free (path);
     else
-      g_queue_push_head (self->priv->files_to_open, g_file_get_parse_name (files[i]));
+      g_queue_push_head (&self->priv->files_to_open, path);
   }
 }
 
@@ -216,7 +187,7 @@ bijiben_init (Bijiben *self)
     G_TYPE_INSTANCE_GET_PRIVATE (self, BIJIBEN_TYPE_APPLICATION, BijibenPriv);
 
   priv->settings = bjb_settings_new ();
-  priv->files_to_open = g_queue_new ();
+  g_queue_init (&priv->files_to_open);
   priv->new_note = FALSE;
   priv->is_loaded = FALSE;
 }
@@ -300,7 +271,7 @@ manager_ready_cb (GObject *source,
       return;
     }
 
-  bijiben_new_window_internal (self, NULL, NULL);
+  bijiben_new_window_internal (self, NULL);
 }
 
 static void
@@ -380,15 +351,13 @@ bijiben_application_local_command_line (GApplication *application,
                                         gchar ***arguments,
                                         gint *exit_status)
 {
+  Bijiben *self;
   gboolean version = FALSE;
   gchar **remaining = NULL;
   GOptionContext *context;
   GError *error = NULL;
   gint argc = 0;
   gchar **argv = NULL;
-  *exit_status = EXIT_SUCCESS;
-  GFile **files;
-  gint idx, len;
 
   const GOptionEntry options[] = {
     { "version", 0, 0, G_OPTION_ARG_NONE, &version,
@@ -400,6 +369,7 @@ bijiben_application_local_command_line (GApplication *application,
     { NULL }
   };
 
+  *exit_status = EXIT_SUCCESS;
 
   context = g_option_context_new (NULL);
   g_option_context_set_summary (context,
@@ -427,15 +397,14 @@ bijiben_application_local_command_line (GApplication *application,
     goto out;
   }
 
-
   /* bijiben_startup */
   g_application_register (application, NULL, &error);
 
   if (error != NULL)
   {
-    /* Translators: this is a fatal error quit message
-     * printed on the command line */
     g_printerr ("%s: %s\n",
+                /* Translators: this is a fatal error quit message
+                 * printed on the command line */
                 _("Could not register the application"),
                 error->message);
     g_error_free (error);
@@ -444,58 +413,21 @@ bijiben_application_local_command_line (GApplication *application,
     goto out;
   }
 
-  if (BIJIBEN_APPLICATION (application)->priv->new_note)
+  self = BIJIBEN_APPLICATION (application);
+
+  if (!self->priv->new_note && remaining != NULL)
   {
-    g_application_open (application, NULL, 0, "");
-    goto out;
+    gchar **args;
+
+    for (args = remaining; *args; args++)
+      if (!bijiben_open_path (self, *args, NULL))
+        g_queue_push_head (&self->priv->files_to_open, g_strdup (*args));
   }
 
-  len = 0;
-  files = NULL;
-
-  /* Convert args to GFiles */
-  if (remaining != NULL)
-  {
-    GFile *file;
-    GPtrArray *file_array;
-
-    file_array = g_ptr_array_new ();
-
-    for (idx = 0; remaining[idx] != NULL; idx++)
-    {
-      file = g_file_new_for_commandline_arg (remaining[idx]);
-      if (file != NULL)
-        g_ptr_array_add (file_array, file);
-    }
-
-    len = file_array->len;
-    files = (GFile **) g_ptr_array_free (file_array, FALSE);
-    g_strfreev (remaining);
-  }
-
-  if (files == NULL)
-  {
-    files = g_malloc0 (sizeof (GFile *));
-    len = 0;
-
-    files[0] = NULL;
-  }
-
-  if (len == 0)
-    goto out;
-
-  /* Invoke "Open" to create new windows */
-  g_application_open (application, files, len, "");
-
-  for (idx = 0; idx < len; idx++)
-  {
-    g_object_unref (files[idx]);
-  }
-
-  g_free (files);
-
  out:
+  g_strfreev (remaining);
   g_option_context_free (context);
+
   return TRUE;
 }
 
@@ -506,7 +438,8 @@ bijiben_finalize (GObject *object)
 
   g_clear_object (&self->priv->manager);
   g_clear_object (&self->priv->settings);
-  g_queue_free (self->priv->files_to_open);
+  g_queue_foreach (&self->priv->files_to_open, (GFunc) g_free, NULL);
+  g_queue_clear (&self->priv->files_to_open);
 
   G_OBJECT_CLASS (bijiben_parent_class)->finalize (object);
 }
